/* Copyright (C) 2025 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergei Zharko [committer] */

/// \file   CbmEventTriggers.h
/// \brief  A structure to store different triggers in parallel to the CbmEvent
/// \since  25.01.2025
/// \author Sergei Zharko <s.zharko@gsi.de>

#pragma once

#include <cstdint>
#include <string>

#if !defined(NO_ROOT) && !XPU_IS_HIP_CUDA
#include <Rtypes.h>  // for ClassDef
#endif

/// \class CbmEventTriggers
/// \brief Class to store different triggers for a given event
class CbmEventTriggers {
 public:
  /// \enum   ETrigger
  /// \brief  Defines a trigger address
  enum class ETrigger : uint8_t
  {
    Lambda = 0b00000001,  ///< Lambda-trigger
    Ks     = 0b00000010   ///< Ks-trigger
  };

  //using Trigger_t = std::underlying_type_t<ETrigger>;
  using Trigger_t = uint8_t;

  /// \brief Default constructor
  CbmEventTriggers() = default;

  /// \brief Copy constructor
  CbmEventTriggers(const CbmEventTriggers&) = default;

  /// \brief Move constructor
  CbmEventTriggers(CbmEventTriggers&&) = default;

  /// \brief Destructor
  ~CbmEventTriggers() = default;

  /// \brief Copy assignment operator
  CbmEventTriggers& operator=(const CbmEventTriggers&) = default;

  /// \brief Move assignment operator
  CbmEventTriggers& operator=(CbmEventTriggers&&) = default;

  /// \brief Sets a trigger
  /// \param key  Trigger key
  void Set(ETrigger key) { fTriggers |= static_cast<Trigger_t>(key); }

  /// \brief Resets a trigger
  /// \param key  Trigger key
  void Reset(ETrigger key) { fTriggers &= ~static_cast<Trigger_t>(key); }

  /// \brief Tests a particular single trigger
  /// \param key  Trigger key
  bool Test(ETrigger key) const { return static_cast<bool>(fTriggers & static_cast<Trigger_t>(key)); }

  /// \brief Tests, if ALL the triggers in the mask are on
  /// \param mask  Trigger mask
  bool TestAll(Trigger_t mask) const { return mask == (fTriggers & mask); }

  /// \brief Tests, if ANY of the triggers in the mask are on
  /// \param mask  Trigger mask
  bool TestAny(Trigger_t mask) const { return static_cast<bool>(fTriggers | mask); }

  /// \brief String representation of the class content
  std::string ToString() const;

 private:
  Trigger_t fTriggers;

#if !defined(NO_ROOT) && !XPU_IS_HIP_CUDA
  ClassDefNV(CbmEventTriggers, 1);
#endif
};
